/*
 * multicast_announce.c
 *
 *  Created on: 05.07.2010
 *      Author: schalli
 */

#include "system.h"

#include "multicast.h"
#include "device.h"
#include <lib/xalloc.h>

#ifdef HAVE_MINGW
#include <w32api.h>
#include <ws2tcpip.h>
#endif

#include <openssl/rand.h>

void send_multicast_announces(void) {
	char *mode = NULL;
	if (get_config_string(lookup_config(config_tree, "LocalAnnounce"), &mode)) {
		if (strcasecmp(mode, "AnnounceOnly") && strcasecmp(mode, "AnnounceAndReceive")) {
			free(mode);
			return;
		}
	} else {
		return;
	}
	free(mode);

	mcpacket_t multicastPacket;
	multicastPacket.type = MCPKT_ANNOUNCE;
	strncpy(multicastPacket.hostname, myself->name, sizeof(multicastPacket.hostname));
	if (netname)
		strncpy(multicastPacket.netname, netname, sizeof(multicastPacket.netname));

	RAND_pseudo_bytes((char*) &mc_challenge, MC_CHALLENGE_SIZE);
	memcpy(multicastPacket.challenge, mc_challenge, MC_CHALLENGE_SIZE);
	gettimeofday(&mc_timestamp, NULL);

	for (int i = 0; i < multicast_sockets; i++) {
		send_multicast_announce(&multicastPacket, i);
	}
}

void send_multicast_announce(const mcpacket_t *multicastPacket, const int i) {
	listen_socket_t *socket = &multicast_socket[i];
	if (socket->sa.sa.sa_family == AF_INET) {
		if (sendto(socket->udp, (char *) multicastPacket, sizeof(mcpacket_t), 0, (struct sockaddr *) &multicastAddr4, sizeof(multicastAddr4)) < 0
				&& !sockwouldblock(sockerrno)) {
			ifdebug(PROTOCOL)
				logger(LOG_WARNING, "Could not announce ipv4 endpoint: %d %s", sockerrno, sockstrerror(sockerrno));
			close_multicast_socket(i);
			return;
		}
	} else if (socket->sa.sa.sa_family == AF_INET6) {
		if (sendto(socket->udp, (char *) multicastPacket, sizeof(mcpacket_t), 0, (struct sockaddr *) &multicastAddr6, sizeof(multicastAddr6)) < 0
				&& !sockwouldblock(sockerrno)) {
			ifdebug(PROTOCOL)
				logger(LOG_WARNING, "Could not announce ipv6 endpoint: %d %s", sockerrno, sockstrerror(sockerrno));
			close_multicast_socket(i);
			return;
		}

	} else {
		// Unknown multicast family
	}
}

void receive_multicast_announce(const int fd, const mcpacket_t *packet, const node_t *n, const sockaddr_t *sa) {
	send_multicast_response(fd, packet, n, sa);
}
