/*
 * multicast_response.c
 *
 *  Created on: 05.07.2010
 *      Author: schalli
 */

 #include "system.h"

#include "multicast.h"
#include "netutl.h"

void send_multicast_response(const int fd, const mcpacket_t *packet, const node_t *n, const sockaddr_t *sa) {
	mcpacket_t response = *packet;
	strncpy((char*) &response.hostname, myself->name, sizeof(response.hostname));
	response.type = MCPKT_RESPONSE;
	response.response.port = htons(atoi(myport));
	sockaddr_t to = *sa;
	to.in.sin_port = htons(multicastPort);

	// sign packet
	set_signature(n, &response);

	// send
	if (sendto(fd, (char *) &response, sizeof(mcpacket_t), 0, &to.sa, SALEN(to.sa)) < 0 && !sockwouldblock(sockerrno)) {
		char *address, *port;
		sockaddr2str(sa, &address, &port);
		ifdebug(PROTOCOL)
			logger(LOG_ERR, "Error sending packet to %s (%s:%s): %s", n->name, address, port, sockstrerror(sockerrno));
		free(address);
		free(port);
	}
}

void receive_multicast_response(node_t *n, const sockaddr_t *sa, const mcpacket_t *packet) {
	struct timeval now;
	gettimeofday(&now, NULL);

	/* latency = response time in ns */
	long latency = (now.tv_sec - mc_timestamp.tv_sec) * 1000;
	latency += (now.tv_usec - mc_timestamp.tv_usec) / 1000;
	latency -= (now.tv_usec < mc_timestamp.tv_usec) ? 1000 : 0;
	latency = (latency > 0) ? latency : 0;

	/*
	 char *address;
	 int port = ntohs(packet->response.port);
	 sockaddr2str(sa, &address, NULL);
	 printf("Got local address for %s, %s%s%i, latency %i\n", n->name, address, ":", port, latency);
	 free(address);
	 */

	sockaddr_t newAddress = *sa;
	newAddress.in.sin_port = packet->response.port;
	update_node_udp_local(n, &newAddress, time(NULL), latency);
	//update_node_udp(n, &newAddress);
}
